package flightsim.logbook;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.*;
import java.io.*;


public class ImportLogBook {

	
	/**
	 * Read a FS9 (in fact FS2000) Log Book file and returns a list of entry.
	 * Do no throw exception on minor parse errors, only when major io error occured
	 * @param fs9lbFile
	 * @return
	 * @throws IOException 
	 * @throws {@link IllegalArgumentException} in case of error
	 */
	public static List<LogEntry> importLogBook(File fs9lbFile) throws IOException {
		return importLogBook(fs9lbFile, "d/M/yy"); //$NON-NLS-1$
	}

	/**
	 * Read a FS9 (in fact FS2000) Log Book file and returns a list of entry.
	 * Do no throw exception on minor parse errors, only when major io error occured
	 * @param fs9lbFile
	 * @param dateFormat date format, for international logbook
	 * @return
	 * @throws IOException 
	 * @throws {@link IllegalArgumentException} in case of error
	 */
	public static List<LogEntry> importLogBook(File fs9lbFile, String dateFormat) throws IOException {
		List<LogEntry> lle = new LinkedList<LogEntry>();
		
		String line;
		BufferedReader br = new BufferedReader(new FileReader(fs9lbFile));
		
		// log book name
		String logBookName = br.readLine();
		if (logBookName == null) throw new IllegalArgumentException(Messages.getString("ImportLogBook.0")); //$NON-NLS-1$
		
		// empty line
		line = br.readLine();
		
		// version
		line = br.readLine();
		if ((line == null) || (!line.startsWith("FS2000")))  //$NON-NLS-1$
			throw new IllegalArgumentException(Messages.getString("ImportLogBook.1")); //$NON-NLS-1$
		
		// skip 2 lines
		line = br.readLine();
		line = br.readLine();
		
		DateFormat df = new SimpleDateFormat(dateFormat);
		// read entries
		while ((line = br.readLine()) != null) {
			if (line.length() < 80) continue;
			if (line.startsWith("TOTALS")) continue; //$NON-NLS-1$
			try {
				LogEntry le = new LogEntry();
				le.setDate(df.parse(line.substring(0, 12).trim()));
				le.setPlaneDesc(line.substring(11, 44).trim());
				le.setDescription(line.substring(44, 68).trim());
				le.setNightTime(Float.parseFloat(line.substring(76,83).trim()));
				le.setInstrumentTime(Float.parseFloat(line.substring(83,90).trim()));
				le.setTotalTime(Float.parseFloat(line.substring(90).trim()));
				lle.add(le);
			} catch (Exception e) {
				// any error here is not fatal. We just skip this entry
				// print error trace for debugging though
				e.printStackTrace();
			}
		}

		br.close();
		
		return lle;
	}
}
