package flightsim.logbook;

import java.io.*;
import java.util.*;



/**
 * TODO: guess what are those unknown fields
 * 
 * @author lc
 * $Id: LogEntry.java,v 1.2 2007-02-03 12:49:31 lc Exp $
 *
 */
public class LogEntry {
	private String from = ""; //$NON-NLS-1$
	private String to = ""; //$NON-NLS-1$
	private String planeReg = ""; //$NON-NLS-1$
	private String planeDesc = ""; //$NON-NLS-1$
	private float totalTime;
	private float nightTime;
	private float instrumentTime;
	private String description = ""; //$NON-NLS-1$
	private long time;				// 64 bits should be enough for everybody (+/- 300 million years with ms precision)
	private Vector<String> landingAirports = new Vector<String>();
	private Vector<Integer> landingTimes = new Vector<Integer>();
	private int acType = AC_TYPE_FIXED_WING;		// default value, since 0 seems unused
	private boolean multimotor;

	/*
	 * AC types, as encoded in the file.
	 */
	public static final int AC_TYPE_GLIDER = 1;
	public static final int AC_TYPE_FIXED_WING = 2;
	public static final int AC_TYPE_AMPHIBIOUS = 3;
	public static final int AC_TYPE_ROTARY = 4;
	
	public static final String[] AC_TYPES_DESCRIPTION = {
		Messages.getString("LogEntry.0"), //$NON-NLS-1$
		Messages.getString("LogEntry.1"), //$NON-NLS-1$
		Messages.getString("LogEntry.2"), //$NON-NLS-1$
		Messages.getString("LogEntry.3"), //$NON-NLS-1$
		Messages.getString("LogEntry.4") //$NON-NLS-1$
	}; 
	
	/* unknown fields (yet!) */
	short unkDataType05 = 0;		// log entry, always 0
	short unkDataType04 = 0;		// flight description, always 0
	short unkDataType03 = 0;		// airports landing tables, always 0
	short unkDataType02 = 0;		// unk from plane reg-desc (seen values: 0, 4000, 8000, C000), probably a bitfield.
									// 4000 seems to be for multimotor ac
	int unkLog01 = 0;				// seems a hash or a random number	
	
	/**
	 * Construct a blank logentry 
	 *
	 */	
	public LogEntry() {
		
	}
	
	
	/**
	 * Construct a logentry. 
	 * @param buffer
	 * @throws IOException 
	 */
	LogEntry(LERandomAccessFile buffer) throws IOException {
		@SuppressWarnings("unused") //$NON-NLS-1$
		long startpos = buffer.getFilePointer();
		
		// read headers
		buffer.readByte();		// type;
		int len = buffer.readByte() & 0xff;		// len
		unkDataType05 = buffer.readShort();
		unkLog01 = buffer.readInt();
		
		// read time
		Calendar cal = Calendar.getInstance();
		cal.set(Calendar.YEAR, buffer.readShort() & 0xffff);
		@SuppressWarnings("unused") byte unk = buffer.readByte(); //$NON-NLS-1$
		cal.set(Calendar.MONTH, (buffer.readByte() & 0xff) - 1);
		cal.set(Calendar.DAY_OF_MONTH, (buffer.readByte() & 0xff));
		cal.set(Calendar.HOUR_OF_DAY, buffer.readByte() & 0xff);
		cal.set(Calendar.MINUTE, buffer.readByte() & 0xff);
		cal.set(Calendar.SECOND, buffer.readByte() & 0xff);
		cal.set(Calendar.MILLISECOND, 0);
		time = cal.getTimeInMillis();
		
		// read from/to
		byte[] temp4 = new byte[4];
		buffer.read(temp4);
		from = new String(temp4, 0, stringLen(temp4));
		buffer.read(temp4);
		to = new String(temp4, 0, stringLen(temp4));
		
		// read flight times
		totalTime = buffer.readFloat();
		nightTime = buffer.readFloat();
		instrumentTime = buffer.readFloat();

		// plane desc-reg
		acType = buffer.readByte() & 0xff;
		unkDataType02 = buffer.readShort();
		int planeDescLen = buffer.readByte()& 0xff;
		byte[] temp10 = new byte[10];
		buffer.read(temp10);
		planeReg = new String(temp10, 0, stringLen(temp10));
		// name
		byte tempD[] = new byte[planeDescLen];
		buffer.read(tempD);
		planeDesc = new String(tempD);
		// mm flag
		multimotor = ((unkDataType02 & 0x4000) != 0);

		while (buffer.getFilePointer() < (startpos + len)) {
			byte type = buffer.readByte();
			switch (type) {
			case 0x02:
				// plane desc-reg
				unkDataType02 = buffer.readShort();
				planeDescLen = buffer.readByte()& 0xff;

				// reg
				temp10 = new byte[10];
				buffer.read(temp10);
				planeReg = new String(temp10, 0, stringLen(temp10));

				// name
				tempD = new byte[planeDescLen];
				buffer.read(tempD);
				planeDesc = new String(tempD);
				
				// mm flag
				multimotor = ((unkDataType02 & 0x4000) != 0);
				break;
			
			case 0x03:
				// Airports
				@SuppressWarnings("unused") //$NON-NLS-1$
				int airportListLen = buffer.readByte() & 0xff;
				unkDataType03 = buffer.readShort();
				int nAp = buffer.readShort()  & 0xffff;
				
				for (int i = 0; i < nAp; i++) {
					buffer.read(temp4);
					String ap = new String(temp4, 0, stringLen(temp4));
					int n = buffer.readByte() & 0xff;
					addLanding(ap, n);
				}
				break;
			
			case 0x04:
				// Flight description
				// FIX 20/11/2006 : trust no length
				//
				int descLen = (buffer.readByte() & 0xff) - 4;
				unkDataType04 = buffer.readShort();
				
				StringBuilder sb = new StringBuilder(descLen);
				int i = 0;
				byte b;
				while ((i < descLen) && ((b = buffer.readByte()) != (byte) 0x5)) {
					sb.append((char)b);
					i++;
				}
				description = sb.toString();
				
				// if we encountered 0x5 in description and before the announced length
				// it's probably a bug in the record. Put back the character
				if (i != descLen) buffer.seek(buffer.getFilePointer()- 1);
				break;
				
			case 0x05:
				// oops, a new entry within our block 
				// put back the char and return
				buffer.seek(buffer.getFilePointer()- 1);
				return;
			}
		}
	}
	
	/**
	 * Because of dump implementsation of String(byte[])
	 * @param barray
	 * @return
	 */
	private int stringLen(byte[] barray) {
		int i = barray.length-1;
		while ((i > 0) && (barray[i] == 0)) i--;
		if (i == 0) return 0;
		return i+1;
	}
	
	private byte[] getPadded(String s, int len) {
		byte[] b = s.getBytes();
		if (b.length != len) {
			byte nb[] = new byte[len];
			Arrays.fill(nb, (byte)0);
			System.arraycopy(b, 0, nb, 0, Math.min(len, b.length));
			return nb;
		}
		return b;
	}
	
	/**
	 * Write entry to buffer
	 * @param buffer
	 * @throws IOException 
	 */
	public void writeEntry(LERandomAccessFile buffer) throws IOException {
		long startpos = buffer.getFilePointer();
		
		// write type 5 entry
		buffer.writeByte((byte) 0x05);
		buffer.writeByte((byte) 0);		// skip len: we will write it later
		buffer.writeShort(unkDataType05);
		buffer.writeInt(unkLog01);
		Calendar cal = Calendar.getInstance();
		cal.setTimeInMillis(time);
		buffer.writeShort((short) cal.get(Calendar.YEAR));
		buffer.writeByte((byte) 0);
		buffer.writeByte((byte) (cal.get(Calendar.MONTH) + 1));
		buffer.writeByte((byte) (cal.get(Calendar.DAY_OF_MONTH)));
		buffer.writeByte((byte) cal.get(Calendar.HOUR_OF_DAY));
		buffer.writeByte((byte) cal.get(Calendar.MINUTE));
		buffer.writeByte((byte) cal.get(Calendar.SECOND));
		
		// from to airports
		buffer.write(getPadded(from, 4));
		buffer.write(getPadded(to, 4));
		
		// write times
		buffer.writeFloat(totalTime);
		buffer.writeFloat(nightTime);
		buffer.writeFloat(instrumentTime);
		
		// write AC type and registrations
		buffer.writeByte((byte) acType);
		byte[] planeDescBs = planeDesc.getBytes();
		if (multimotor) unkDataType02 |= 0x4000;
		buffer.writeShort(unkDataType02);
		buffer.writeByte((byte) planeDescBs.length);
		buffer.write(getPadded(planeReg, 10));
		buffer.write(planeDescBs);
		
		// write type 3 (landing tables) if any
		if ((landingAirports != null) && (landingAirports.size() > 0)) {
			int nAp = landingAirports.size();
			buffer.writeByte((byte) 0x03);
			int airportsBlocLen = 5*nAp + 6;
			buffer.writeByte((byte)airportsBlocLen);
			buffer.writeShort(unkDataType03);
			buffer.writeShort((short)nAp);
			for (int i = 0; i < nAp; i++) {
				buffer.write(getPadded(landingAirports.get(i), 4));
				buffer.writeByte((byte) landingTimes.get(i).intValue());
			}
		}
		
		// write type 4 (description) block if any
		// re writing this block "well formed" will fix a lot
		// of corrupted logbooks
		if ((description != null) && (description.length() > 0)) {
			byte[] descriptionBs = description.getBytes();
			buffer.writeByte((byte) 0x4);
			buffer.writeByte((byte)(descriptionBs.length + 4));
			buffer.writeShort(unkDataType04);
			buffer.write(descriptionBs);
		}
		
		// write len
		long curpos = buffer.getFilePointer();
		buffer.seek(startpos+1);
		buffer.writeByte((byte)(curpos - startpos));
		buffer.seek(curpos);
	}

	public String getDescription() {
		return description;
	}

	public String getFrom() {
		return from;
	}

	public float getInstrumentTime() {
		return instrumentTime;
	}

	public float getDayTime() {
		return totalTime - nightTime;
	}
	
	public int numberLandings() {
		if (landingTimes == null) return 0;
		return landingTimes.size();
	}
	
	public String getLandingAirport(int i) {
		return landingAirports.get(i);
	}

	public int getLandingTimes(int i) {
		return landingTimes.get(i);
	}

	public float getNightTime() {
		return nightTime;
	}

	public String getPlaneDesc() {
		return planeDesc;
	}

	public String getPlaneReg() {
		return planeReg;
	}

	public String getTo() {
		return to;
	}

	public float getTotalTime() {
		return totalTime;
	}
	
	public long getTimeMillis() {
		return time;
	}
	
	public Date getDate() {
		return new Date(time);
	}

	public Date getTime() {
		return new Date(time);
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public void setFrom(String from) {
		this.from = from;
	}

	public void setInstrumentTime(float instrumentTime) {
		this.instrumentTime = instrumentTime;
	}

	public void setNightTime(float nightTime) {
		this.nightTime = nightTime;
	}

	public void setPlaneDesc(String planeDesc) {
		this.planeDesc = planeDesc;
	}

	public void setPlaneReg(String planeReg) {
		this.planeReg = planeReg;
	}

	public void setTimeMillis(long time) {
		this.time = time;
	}
	
	public void setDate(Date d) {
		this.time = d.getTime();
	}
	
	public void setTo(String to) {
		this.to = to;
	}

	public void setTotalTime(float totalTime) {
		this.totalTime = totalTime;
	}

	public void setLandingAirport(int i, String s) {
		if (i > landingAirports.size()) return;
		landingAirports.set(i,s);
	}

	public void setLandingTimes(int i, int n) {
		if (i > landingTimes.size()) return;
		landingTimes.set(i,n);
	}
	
	void print() {
		System.out.println("From " + getFrom() + " to " + getTo()); //$NON-NLS-1$ //$NON-NLS-2$
		System.out.println("Date : " + getDate()); //$NON-NLS-1$
		System.out.println("Total time " + getTotalTime() + " night " + getNightTime() + " instr " + getInstrumentTime()); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		System.out.println("Plane : " + getPlaneDesc() + " reg " + getPlaneReg()); //$NON-NLS-1$ //$NON-NLS-2$
		System.out.println("Description : " + getDescription()); //$NON-NLS-1$
		for (int i = 0; i < numberLandings(); i++)  {
			System.out.println("On " + getLandingAirport(i) + " " + getLandingTimes(i)); //$NON-NLS-1$ //$NON-NLS-2$
		}
		System.out.println();

	}

	public Collection<? extends String> getLandingAirports() {
		return landingAirports;
	}

	public Collection<? extends Integer> getLandingTimes() {
		return landingTimes;
	}

	public void clearLandingTable() {
		landingTimes.clear();
		landingAirports.clear();
	}

	public void addLanding(String ap, int n) {
		landingAirports.add(ap);
		landingTimes.add(n);
	}

	/**
	 * Gets the aircraft type
	 * @see AC_TYPE_AMPHIBIOUS
	 * @see AC_TYPE_FIXED_WING
	 * @return
	 */
	public int getAircraftType() {
		return acType;
	}
	
	/**
	 * Sets the aircraft type
	 * @see AC_TYPE_AMPHIBIOUS
	 * @see AC_TYPE_FIXED_WING
	 * @return
	 */
	public void setAircraftType(int acType) {
		this.acType = acType;
	}
	
	public boolean isMultimotor() {
		return multimotor;
	}
	
	public void setMultimotor(boolean multimotor) {
		this.multimotor = multimotor;
	}
	
	
}
