/*
 * ExportDialog.java
 *
 * Created on February 2, 2007, 4:20 PM
 */

package flightsim.logbook.gui;

import java.io.*;
import java.text.*;
import java.util.*;

import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.table.AbstractTableModel;

import flightsim.logbook.LogBook;
import flightsim.logbook.gui.exportfilters.ExportField;
import flightsim.logbook.gui.exportfilters.Exporter;
import flightsim.logbook.gui.exportfilters.PrintExporter;


/**
 *
 * @author  lc
 */
@SuppressWarnings("serial") //$NON-NLS-1$
public class ExportDialog extends JDialog {
    
	private JPanel mainPanel;
	private LogBook lb;
	private List<ExportField> useFieldsList;
	private ExportFieldsTableModel allFieldsModel;
	private ExportFieldsTableModel useFieldsModel;
	private boolean doPrint;
	
	private static final String[] DEFAULT_FIELDS = {
		"Date/Time (medium)", //$NON-NLS-1$
		"From/To airport ICAO", //$NON-NLS-1$
		"Plane description", //$NON-NLS-1$
		"Plane registration", //$NON-NLS-1$
		"Flight time (short)", //$NON-NLS-1$
		"Flight description" //$NON-NLS-1$
	};
	
    /** Creates new form ExportDialog */
    public ExportDialog(JFrame jf) {
    	super(jf, true);
    	setSize(559, 450);
    	setTitle(Messages.getString("ExportDialog.7")); //$NON-NLS-1$
    	mainPanel = new JPanel();
        initComponents();
		setContentPane(mainPanel);
        
        // fill table contents
		allFieldsModel = new ExportFieldsTableModel(Exporter.getExportableFields(), Messages.getString("ExportDialog.8")); //$NON-NLS-1$
		allFieldsTable.setModel(allFieldsModel);
		
		useFieldsList = new LinkedList<ExportField>();
		useFieldsModel = new ExportFieldsTableModel(useFieldsList, Messages.getString("ExportDialog.9")); //$NON-NLS-1$
		usedFieldsTable.setModel(useFieldsModel);
        
		// default selection
		for (String s : DEFAULT_FIELDS) {
			ExportField ef = Exporter.getExportField(s);
			if (ef != null) useFieldsModel.add(ef);
		}
    }
    
    void setPrintDialog(boolean print) {
    	doPrint = print;
    	outFileTextField.setEnabled(!print);
    	chooseFileButton.setEnabled(!print);
    	jLabel1.setEnabled(!print);
    	jLabel2.setEnabled(!print);
    	formatSelectionComboBox.setEnabled(!print);
    	if (print) {
    		formatSelectionComboBox.setModel(
    				new javax.swing.DefaultComboBoxModel(
    						new String[] { Messages.getString("ExportDialog.6") })); //$NON-NLS-1$
    	} else {
            formatSelectionComboBox.setModel(
            		new javax.swing.DefaultComboBoxModel(
            				new String[] { 
            						Messages.getString("ExportDialog.18"),  //$NON-NLS-1$
            						Messages.getString("ExportDialog.19") }));  //$NON-NLS-1$
    	}
   }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {
        jScrollPane1 = new javax.swing.JScrollPane();
        usedFieldsTable = new javax.swing.JTable();
        jSeparator1 = new javax.swing.JSeparator();
        upButton = new javax.swing.JButton();
        dnButton = new javax.swing.JButton();
        addButton = new javax.swing.JButton();
        delButton = new javax.swing.JButton();
        jScrollPane2 = new javax.swing.JScrollPane();
        allFieldsTable = new javax.swing.JTable();
        jLabel1 = new javax.swing.JLabel();
        jLabel2 = new javax.swing.JLabel();
        jSeparator2 = new javax.swing.JSeparator();
        okButton = new javax.swing.JButton();
        cancelButton = new javax.swing.JButton();
        formatSelectionComboBox = new javax.swing.JComboBox();
        outFileTextField = new javax.swing.JTextField();
        chooseFileButton = new javax.swing.JButton();

        jScrollPane1.setViewportView(usedFieldsTable);

        upButton.setText(Messages.getString("ExportDialog.10")); //$NON-NLS-1$
        upButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                upButtonActionPerformed(evt);
            }
        });

        dnButton.setText(Messages.getString("ExportDialog.11")); //$NON-NLS-1$
        dnButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                dnButtonActionPerformed(evt);
            }
        });

        addButton.setText(Messages.getString("ExportDialog.12")); //$NON-NLS-1$
        addButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                addButtonActionPerformed(evt);
            }
        });

        delButton.setText(Messages.getString("ExportDialog.13")); //$NON-NLS-1$
        delButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                delButtonActionPerformed(evt);
            }
        });
        allFieldsTable.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                allFieldsTableMouseClicked(evt);
            }
        });
        usedFieldsTable.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                useFieldsTableMouseClicked(evt);
            }
        });

        jScrollPane2.setViewportView(allFieldsTable);

        jLabel1.setText(Messages.getString("ExportDialog.14")); //$NON-NLS-1$

        jLabel2.setText(Messages.getString("ExportDialog.15")); //$NON-NLS-1$

        okButton.setText(Messages.getString("ExportDialog.16")); //$NON-NLS-1$
        okButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                okButtonActionPerformed(evt);
            }
        });

        cancelButton.setText(Messages.getString("ExportDialog.17")); //$NON-NLS-1$
        cancelButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cancelButtonActionPerformed(evt);
            }
        });

        formatSelectionComboBox.setModel(new javax.swing.DefaultComboBoxModel(new String[]  { Messages.getString("ExportDialog.18"), Messages.getString("ExportDialog.19") })); //$NON-NLS-1$ //$NON-NLS-2$

        outFileTextField.setText(""); //$NON-NLS-1$

        try {
            chooseFileButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/flightsim/logbook/gui/Open16.gif"))); //$NON-NLS-1$
        } catch (NullPointerException npe) {}
        chooseFileButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                chooseFileButtonActionPerformed(evt);
            }
        });

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(mainPanel);
        mainPanel.setLayout(layout);
        layout.setHorizontalGroup(
                layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                .add(layout.createSequentialGroup()
                    .addContainerGap()
                    .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                        .add(jSeparator1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 535, Short.MAX_VALUE)
                        .add(layout.createSequentialGroup()
                            .add(jScrollPane1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 220, Short.MAX_VALUE)
                            .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                            .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                .add(org.jdesktop.layout.GroupLayout.TRAILING, upButton, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 83, Short.MAX_VALUE)
                                .add(org.jdesktop.layout.GroupLayout.TRAILING, dnButton, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 83, Short.MAX_VALUE)
                                .add(org.jdesktop.layout.GroupLayout.TRAILING, addButton, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 83, Short.MAX_VALUE)
                                .add(org.jdesktop.layout.GroupLayout.TRAILING, delButton, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                            .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                            .add(jScrollPane2, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 220, Short.MAX_VALUE))
                        .add(layout.createSequentialGroup()
                            .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                .add(jLabel1)
                                .add(jLabel2))
                            .add(48, 48, 48)
                            .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
                                    .add(outFileTextField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 353, Short.MAX_VALUE)
                                    .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                    .add(chooseFileButton, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 35, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                                .add(formatSelectionComboBox, 0, 394, Short.MAX_VALUE)))
                        .add(org.jdesktop.layout.GroupLayout.TRAILING, jSeparator2, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 535, Short.MAX_VALUE)
                        .add(layout.createSequentialGroup()
                            .add(okButton, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 66, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                            .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                            .add(cancelButton)))
                    .addContainerGap())
            );

            layout.linkSize(new java.awt.Component[] {addButton, delButton, dnButton, upButton}, org.jdesktop.layout.GroupLayout.HORIZONTAL);

            layout.linkSize(new java.awt.Component[] {cancelButton, okButton}, org.jdesktop.layout.GroupLayout.HORIZONTAL);

            layout.setVerticalGroup(
                layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                .add(layout.createSequentialGroup()
                    .addContainerGap()
                    .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                        .add(layout.createSequentialGroup()
                            .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                .add(layout.createSequentialGroup()
                                    .add(upButton, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 26, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                    .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                    .add(dnButton, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 26, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                    .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                    .add(addButton, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 26, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                    .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                    .add(delButton, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 28, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                    .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 184, Short.MAX_VALUE))
                                .add(jScrollPane1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 314, Short.MAX_VALUE))
                            .add(9, 9, 9))
                        .add(layout.createSequentialGroup()
                            .add(jScrollPane2, 0, 314, Short.MAX_VALUE)
                            .add(9, 9, 9)))
                    .add(jSeparator1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 10, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                    .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                        .add(jLabel1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 25, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .add(formatSelectionComboBox, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                    .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                    .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                        .add(chooseFileButton, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 17, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .add(outFileTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .add(jLabel2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 25, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                    .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                    .add(jSeparator2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 20, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                    .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                        .add(okButton)
                        .add(cancelButton))
                    .addContainerGap())
            );

            layout.linkSize(new java.awt.Component[] {chooseFileButton, formatSelectionComboBox, outFileTextField}, org.jdesktop.layout.GroupLayout.VERTICAL);

            layout.linkSize(new java.awt.Component[] {addButton, delButton, dnButton, upButton}, org.jdesktop.layout.GroupLayout.VERTICAL);

    }// </editor-fold>                        

    private String choosedFormat() {
    	Object o = formatSelectionComboBox.getSelectedItem();
    	if (o != null && o instanceof String) {
			String s = (String) o;
			int ip = s.indexOf(' ');
			if (ip != -1) s = s.substring(0, ip);
			return s.toLowerCase();
			
		}
    	return null;
    }
    
    private String selectOutputFile() {
    	JFileChooser jf = new JFileChooser();
    	jf.setFileFilter(new ExtensionFileFilter(choosedFormat(), formatSelectionComboBox.getSelectedItem().toString()));
    	int sel = jf.showOpenDialog(this);
    	if (sel == JFileChooser.APPROVE_OPTION) {
    		if (jf.getSelectedFile() != null) {
        		String sf = jf.getSelectedFile().toString();
        		return sf;
    		}
    	}
    	return null;
    }
    
    private void chooseFileButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_chooseFileButtonActionPerformed
    	String sf = selectOutputFile();
    	if (sf != null) 
    		outFileTextField.setText(sf);
    }//GEN-LAST:event_chooseFileButtonActionPerformed

    private void cancelButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cancelButtonActionPerformed
    	setVisible(false);
    }//GEN-LAST:event_cancelButtonActionPerformed

    private void okButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_okButtonActionPerformed
    	//
    	// big export action
    	//
    	setVisible(false);
    	if (lb == null) return;
    	
		if (doPrint) {
			PrintExporter e = Exporter.getPrintExporter(useFieldsList, lb);
			e.doPrint();
			
		} else {
	    	// ok go
	    	String exportFormat = choosedFormat();
	    	
	    	String outFileName = outFileTextField.getText();
	    	if (outFileName == null || "".equals(outFileName)) { //$NON-NLS-1$
	    		// reask 
	    		outFileName = selectOutputFile();
	    	}
	    	
	    	if (outFileName == null) return;
	    	
			Exporter e = null;
	    	if ("html".equals(exportFormat)) { //$NON-NLS-1$
	    		e = Exporter.getHTMLExporter(useFieldsList);
	    	} else if ("csv".equals(exportFormat)) { //$NON-NLS-1$
	    		e = Exporter.getCSVExporter(useFieldsList);
	    	}
	     	try {
				e.writeLogBook(lb, new File(outFileName));
				JOptionPane.showMessageDialog(this, MessageFormat.format(Messages.getString("ExportDialog.26"), outFileName), Messages.getString("ExportDialog.27"), JOptionPane.INFORMATION_MESSAGE); //$NON-NLS-1$ //$NON-NLS-2$
			} catch (Exception e1) {
				JOptionPane.showMessageDialog(this, MessageFormat.format(Messages.getString("ExportDialog.28"), e1.getMessage()), Messages.getString("ExportDialog.29"), JOptionPane.ERROR_MESSAGE); //$NON-NLS-1$ //$NON-NLS-2$
				e1.printStackTrace();
			}
		}
    	
    }//GEN-LAST:event_okButtonActionPerformed

    private void delButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_delButtonActionPerformed
    	int sel = usedFieldsTable.getSelectedRow();
    	if (sel != -1) {
    		useFieldsModel.remove(sel);
    	}
    }//GEN-LAST:event_delButtonActionPerformed

    private void addButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_addButtonActionPerformed
    	int sel = allFieldsTable.getSelectedRow();
    	if (sel != -1) {
    		ExportField ef = allFieldsModel.get(sel);
    		useFieldsModel.add(ef);
    	}
    }//GEN-LAST:event_addButtonActionPerformed

    private void dnButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_dnButtonActionPerformed
    	int sel = usedFieldsTable.getSelectedRow();
    	if (sel != -1 && sel < (useFieldsModel.getRowCount()-1)) {
    		useFieldsModel.moveDown(sel);
     		usedFieldsTable.getSelectionModel().setSelectionInterval(sel+1, sel+1);
    	}
    }//GEN-LAST:event_dnButtonActionPerformed

    private void upButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_upButtonActionPerformed
    	int sel = usedFieldsTable.getSelectedRow();
    	if (sel != -1 && sel > 0) {
    		useFieldsModel.moveUp(sel);
    		usedFieldsTable.getSelectionModel().setSelectionInterval(sel-1, sel-1);
    	}
    }//GEN-LAST:event_upButtonActionPerformed
    
    private void allFieldsTableMouseClicked(java.awt.event.MouseEvent evt) {
    	if (evt.getClickCount() == 2) {
        	int sel = allFieldsTable.getSelectedRow();
        	if (sel != -1) {
        		ExportField ef = allFieldsModel.get(sel);
        		useFieldsModel.add(ef);
        	}
    	}
    }
    private void useFieldsTableMouseClicked(java.awt.event.MouseEvent evt) {
    	if (evt.getClickCount() == 2) {
        	int sel = usedFieldsTable.getSelectedRow();
        	if (sel != -1) {
        		useFieldsModel.remove(sel);
        	}
    	}
    }

    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton addButton;
    protected javax.swing.JTable allFieldsTable;
    private javax.swing.JButton cancelButton;
    private javax.swing.JButton chooseFileButton;
    private javax.swing.JButton delButton;
    private javax.swing.JButton dnButton;
    private javax.swing.JComboBox formatSelectionComboBox;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JSeparator jSeparator1;
    private javax.swing.JSeparator jSeparator2;
    private javax.swing.JButton okButton;
    private javax.swing.JTextField outFileTextField;
    private javax.swing.JButton upButton;
    protected javax.swing.JTable usedFieldsTable;
    // End of variables declaration//GEN-END:variables

	public void setLogBook(LogBook currentLogBook) {
		this.lb = currentLogBook;
		
	}
    
}


@SuppressWarnings("serial") //$NON-NLS-1$
class ExportFieldsTableModel extends AbstractTableModel {
	
	private List<ExportField> lef;
	private String name;
	
	ExportFieldsTableModel(List<ExportField> lef, String s) {
		this.lef = lef;
		this.name = s;
	}
	
	public int getColumnCount() {
		return 1;
	}

	public int getRowCount() {
		return lef.size();
	}

	public Object getValueAt(int rowIndex, int columnIndex) {
		ExportField ef = lef.get(rowIndex);
		return ef;
	}
	
	public ExportField get(int i) {
		return lef.get(i);
	}
	
	@Override
	public String getColumnName(int column) {
		return name;
	}
	
	@Override
	public boolean isCellEditable(int rowIndex, int columnIndex) {
		return false;
	}
	
	public void add(ExportField ef) {
		lef.add(ef);
		fireTableRowsInserted(getRowCount(), getRowCount());
	}
	
	public void moveUp(int index) {
		if (index < (lef.size() -1)) {
			Collections.swap(lef, index-1, index);
			fireTableRowsUpdated(index-1, index);
		}
	}

	public void moveDown(int index) {
		if (index > 0) {
			Collections.swap(lef, index+1, index);
			fireTableRowsUpdated(index+1, index);
		}
	}
	
	public void remove(int index) {
		lef.remove(index);
		fireTableRowsDeleted(index, index);
	}

}
