/**
 * 
 */
package flightsim.logbook.gui;

import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.text.ParseException;

import javax.swing.text.DefaultFormatter;


@SuppressWarnings("serial") //$NON-NLS-1$
/**
 * This class formats a float (indication a duration in hours) into a
 * string in format HH:mm
 */
public class FloatDurationFormatter extends DefaultFormatter {
	private static final NumberFormat FMT = new DecimalFormat("00"); //$NON-NLS-1$
	
	FloatDurationFormatter() {
		setOverwriteMode(false);
	}
	
	@Override
	public Object stringToValue(String string) throws ParseException {
		if (string != null) {
			int ip = string.indexOf(':');
			if (ip == -1) throw new ParseException(Messages.getString("FloatDurationFormatter.0"), 0); //$NON-NLS-1$
			try {
				int hPart = Integer.parseInt(string.substring(0, ip));
				int mPart = Integer.parseInt(string.substring(ip+1));
				float f = hPart;
				f += ((float)mPart) / 60.0f;
				return f;
				
			} catch (NumberFormatException nfe) {
				throw new ParseException(nfe.getLocalizedMessage(), 0);
			}
		}
		return null;
	}
	
	@Override
	public String valueToString(Object value) throws ParseException {
		if (value != null && (value instanceof Float)) {	
			float val = (Float) value;
			int hPart = (int) Math.floor(val);
			int mPart = (int) Math.round((val - hPart) * 60.0f);
			// just in case
			if (mPart == 60) {
				hPart++;
				mPart = 0;
			}
			return hPart + ":" + FMT.format(mPart); //$NON-NLS-1$
		}
		return null;
	}
	
}