package flightsim.logbook.gui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.GridLayout;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.text.DateFormat;
import java.text.MessageFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JViewport;
import javax.swing.KeyStroke;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.filechooser.FileFilter;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.DefaultTableCellRenderer;


import flightsim.logbook.ImportLogBook;
import flightsim.logbook.LogBook;
import flightsim.logbook.LogEntry;

/**
 * GUI
 * TODO: print (using exportdialog)
 * 
 * @author lc
 * $Id: LogBookManager.java,v 1.3 2007-02-03 13:37:21 lc Exp $
 */
public class LogBookManager {

	public static final String VERSION = "0.3"; //$NON-NLS-1$

	private JFrame frame; 
	private JTable logTable;
	private LogBook currentLogBook = null;
	
	private JMenuItem fileSaveItem;
	private JMenuItem fileExportItem;
	private JMenuItem filePrintItem;
	private JMenuItem editDelItem;
	private JMenuItem editAddItem;
	private JMenuItem editImportFS9Item;
	
	private boolean modified = false;
	private LogBookTableModel tableModel;
	private TableSorter tableModelSorted;
	
	private LogEntryEditor logEntryEditor;
	private ImportOptionsDialog importOptionsDialog;
	private ExportDialog exportDialog;
	
	public LogBookManager() {
		// construct gui
		frame = new JFrame(Messages.getString("LogBookManager.0")); //$NON-NLS-1$
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.setSize(800, 600);
		frame.getContentPane().setLayout(new BorderLayout());
		frame.addWindowListener(new WindowListener() {
			public void windowActivated(WindowEvent e) {}
			public void windowClosing(WindowEvent e) {
				quitAction();
			}
			public void windowDeactivated(WindowEvent e) {}
			public void windowDeiconified(WindowEvent e) {}
			public void windowIconified(WindowEvent e) {}
			public void windowOpened(WindowEvent e) {}
			public void windowClosed(WindowEvent e) {}
			});
		
		// menu items
		JMenuBar menuBar = new JMenuBar();
		JMenu fileMenu = new JMenu(Messages.getString("LogBookManager.1")); //$NON-NLS-1$
		fileMenu.setMnemonic('F');
		
		JMenuItem fileOpenItem = new JMenuItem(Messages.getString("LogBookManager.2")); //$NON-NLS-1$
		fileOpenItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O, 0));
		fileOpenItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				openAction();
			}});
		fileMenu.add(fileOpenItem);
		
		fileSaveItem = new JMenuItem(Messages.getString("LogBookManager.3")); //$NON-NLS-1$
		fileSaveItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, InputEvent.CTRL_MASK));
		fileSaveItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				saveAction();
			}});
		fileSaveItem.setEnabled(false);
		fileMenu.add(fileSaveItem);
		
		fileExportItem = new JMenuItem(Messages.getString("LogBookManager.4")); //$NON-NLS-1$
		fileExportItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_E, 0));
		fileExportItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				exportAction();
			}});
		fileExportItem.setEnabled(false);
		fileMenu.add(fileExportItem);
		
		
		filePrintItem = new JMenuItem(Messages.getString("LogBookManager.49")); //$NON-NLS-1$
		filePrintItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_P, 0));
		filePrintItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				printAction();
			}});
		filePrintItem.setEnabled(false);
		fileMenu.add(filePrintItem);
		
		fileMenu.addSeparator();
		JMenuItem fileQuItem = new JMenuItem(Messages.getString("LogBookManager.5")); //$NON-NLS-1$
		fileQuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Q, 0));
		fileQuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				quitAction();
			}});
		fileMenu.add(fileQuItem);
		menuBar.add(fileMenu);
		
		JMenu editMenu = new JMenu(Messages.getString("LogBookManager.6")); //$NON-NLS-1$
		editMenu.setMnemonic('E');
		
		editDelItem = new JMenuItem(Messages.getString("LogBookManager.7")); //$NON-NLS-1$
		editDelItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_DELETE, 0));
		editDelItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				delItemAction();
			}});
		editDelItem.setEnabled(false);
		editMenu.add(editDelItem);
		
		editAddItem = new JMenuItem(Messages.getString("LogBookManager.8")); //$NON-NLS-1$
		editAddItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_INSERT, 0));
		editAddItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				addItemAction();
			}});
		editAddItem.setEnabled(false);
		editMenu.add(editAddItem);

		editImportFS9Item = new JMenuItem(Messages.getString("LogBookManager.9")); //$NON-NLS-1$
		editImportFS9Item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_I, 0));
		editImportFS9Item.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				importAction();
			}});
		editImportFS9Item.setEnabled(false);
		editMenu.add(editImportFS9Item);

		
		menuBar.add(editMenu);
		
		JMenu aboutMenu = new JMenu(Messages.getString("LogBookManager.10")); //$NON-NLS-1$
		JMenuItem aboutMenuItem = new JMenuItem(Messages.getString("LogBookManager.11")); //$NON-NLS-1$
		aboutMenuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				JOptionPane.showMessageDialog(frame, 
						MessageFormat.format(Messages.getString("LogBookManager.12") + //$NON-NLS-1$
								Messages.getString("LogBookManager.13") + //$NON-NLS-1$
								Messages.getString("LogBookManager.14"), VERSION), //$NON-NLS-1$
						Messages.getString("LogBookManager.15"), JOptionPane.INFORMATION_MESSAGE);  //$NON-NLS-1$
			}});
		aboutMenu.add(aboutMenuItem);
		menuBar.add(aboutMenu);
		
		frame.setJMenuBar(menuBar);
		
		// log entry editor
		logEntryEditor = new LogEntryEditor(frame);
		
		// a widget for asking import optiongs
		importOptionsDialog = new ImportOptionsDialog(frame);
		
		// export
		exportDialog = new ExportDialog(frame);
		exportDialog.setPrintDialog(false);
		
		// log table
		// a sorter model from swing's doc
		tableModel = new LogBookTableModel();
		tableModelSorted = new TableSorter(tableModel);
		logTable = new JTable(tableModelSorted);
		tableModelSorted.setTableHeader(logTable.getTableHeader());
			
		// set some max width for fixed-width columns so that descriptions will
		// have more space
		logTable.getColumnModel().getColumn(0).setMaxWidth(60);	// date
		logTable.getColumnModel().getColumn(1).setMaxWidth(60);	// time
		logTable.getColumnModel().getColumn(2).setMaxWidth(50);	// from
		logTable.getColumnModel().getColumn(3).setMaxWidth(50);	// to
		logTable.getColumnModel().getColumn(4).setMaxWidth(80); // plane reg
		logTable.getColumnModel().getColumn(6).setMaxWidth(50); // times
		logTable.getColumnModel().getColumn(7).setMaxWidth(50);
		logTable.getColumnModel().getColumn(8).setMaxWidth(50);
			
		// some renderers and editors
		logTable.setDefaultRenderer(Float.class, new DurationRenderer());
		logTable.getColumnModel().getColumn(0).setCellRenderer(new SimpleDateRenderer(DateFormat.getDateInstance(DateFormat.SHORT)));
		logTable.getColumnModel().getColumn(1).setCellRenderer(new SimpleDateRenderer(DateFormat.getTimeInstance(DateFormat.SHORT)));
		
		logTable.getSelectionModel().addListSelectionListener(new ListSelectionListener() {

			public void valueChanged(ListSelectionEvent e) {
				if (e.getValueIsAdjusting() ) return;
				if (e.getFirstIndex() == -1) editDelItem.setEnabled(false);
				else editDelItem.setEnabled(true);
				
			}});
		
		// add a mouse listener that watch double click to launch edit window
		logTable.addMouseListener(new MouseListener(){

			public void mouseClicked(MouseEvent e) {
				if (e.getClickCount() == 2) {
					popupLogEntryEditor();
				}
			}

			public void mouseEntered(MouseEvent e) {}
			public void mouseExited(MouseEvent e) {}
			public void mousePressed(MouseEvent e) {}
			public void mouseReleased(MouseEvent e) {}
			});
		// selection mode
		logTable.getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
			
		// an listener to see changes in table
		tableModel.addTableModelListener(new TableModelListener() {
			public void tableChanged(TableModelEvent e) {
				setModified(true);
			}});
		
		frame.getContentPane().add(new JScrollPane(logTable), BorderLayout.CENTER);
	}
	
	/**
	 * Gui action called to open a file
	 *
	 */
	private void openAction() {
		// ask for a file
		JFileChooser fc = new JFileChooser();
		fc.setMultiSelectionEnabled(false);
		fc.setFileFilter(new FileFilter() {
			public boolean accept(File f) {
				if (f.isDirectory()) return true;
				if (f.getName().equalsIgnoreCase("Logbook.BIN")) return true; //$NON-NLS-1$
				return false;
			}
			public String getDescription() {
				return Messages.getString("LogBookManager.16"); //$NON-NLS-1$
			}});
		int option = fc.showOpenDialog(frame);
		if (option == JFileChooser.APPROVE_OPTION) {
			File f = fc.getSelectedFile();
			// try opening file
			openLogBook(f);
		}
	}
	
	/**
	 * Open a book, setup table models and activate menu entries
	 * @param f
	 */
	private void openLogBook(File f) {
		try {
			currentLogBook = new LogBook(f);
			// update window title
			tableModel.setLogBook(currentLogBook);
			exportDialog.setLogBook(currentLogBook);
			setModified(false);
			fileExportItem.setEnabled(true);
			filePrintItem.setEnabled(true);
			fileSaveItem.setEnabled(true);
			editAddItem.setEnabled(true);
			editImportFS9Item.setEnabled(true);
		} catch (Exception e) {
			e.printStackTrace();
			JOptionPane.showMessageDialog(frame, 
					MessageFormat.format(Messages.getString("LogBookManager.17"), e.getMessage()), //$NON-NLS-1$
					Messages.getString("LogBookManager.18"), JOptionPane.ERROR_MESSAGE); //$NON-NLS-1$
		}
	}
	
	/**
	 * reset window title according to the flag.
	 * @param b
	 */
	private void setModified(boolean b) {
		if (currentLogBook == null) return;
		modified = b;
		if (b == true) {
			frame.setTitle(Messages.getString("LogBookManager.19") + currentLogBook.getFile().getAbsolutePath() + ") *");  //$NON-NLS-1$ //$NON-NLS-2$
		} else {
			frame.setTitle(Messages.getString("LogBookManager.21") + currentLogBook.getFile().getAbsolutePath() + ")");  //$NON-NLS-1$ //$NON-NLS-2$
		}
	}
	
	private boolean popupLogEntryEditor() {
		int row = logTable.getSelectedRow();
		if (row != -1) {
			return popupLogEntryEditor(tableModelSorted.modelIndex(row));
		}
		return false;
	}
	
	private boolean popupLogEntryEditor(int tableModelRow) {
		if (tableModelRow != -1) {
			LogEntry le = tableModel.getRow(tableModelRow);
			if (le != null) {
				logEntryEditor.setLogEntry(le);
				logEntryEditor.setVisible(true);
				// since log entry editor is modal, we can wait for it
				boolean modif = logEntryEditor.isModified();
				if (modif) tableModel.fireTableRowsUpdated(tableModelRow, tableModelRow);
				return modif;
			}
		}
		return false;
	}

	/**
	 * Called by GUI on "save" menu choice
	 *
	 */
	private void saveAction() {
		if (currentLogBook == null) return;
		try {
			currentLogBook.write();
			setModified(false);
		} catch (Exception e) {
			JOptionPane.showMessageDialog(frame, 
					MessageFormat.format(Messages.getString("LogBookManager.23"), e.getMessage()), //$NON-NLS-1$
					Messages.getString("LogBookManager.24"), JOptionPane.ERROR_MESSAGE); //$NON-NLS-1$
			e.printStackTrace();
		}
	}
	
	/**
	 * Called by GUI on "quit" menu choice
	 *
	 */
	private void quitAction() {
		if (modified) {
			int choice = JOptionPane.showConfirmDialog(frame,
					Messages.getString("LogBookManager.25"), //$NON-NLS-1$
					Messages.getString("LogBookManager.26"), //$NON-NLS-1$
					JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE);
			if (choice == JOptionPane.YES_OPTION) {
				saveAction();
			}
		}
		System.exit(0);
	}
	
	/**
	 * Called by GUI on "delete entry" menu choice
	 *
	 */
	private void delItemAction() {
		if (currentLogBook == null) return;
		
		int rowIdx = logTable.getSelectedRow();
		if (rowIdx == -1) return;
		rowIdx = tableModelSorted.modelIndex(rowIdx);
		tableModel.delEntry(rowIdx);
		
	}

	/**
	 * Called by GUI on "add entry" menu choice
	 *
	 */
	private void addItemAction() {
		if (currentLogBook == null) return;
		
		// add a new row
		int pos = tableModel.addRow();
		
		// and scroll to it
		scrollToCenter(logTable, tableModelSorted.getModelToView()[pos], 0);

		// open the editing
		if (!popupLogEntryEditor(pos)) {
			// called cancel on the logentry editor
			tableModel.delEntry(pos);
		}
	}
	
	/**
	 * Called by GUI on "import" menu choice
	 *
	 */
	private void importAction() {
		if (currentLogBook == null) return;
		
		// ask for a file
		JFileChooser fc = new JFileChooser();
		fc.setMultiSelectionEnabled(false);
		fc.setFileFilter(new ExtensionFileFilter("log", Messages.getString("LogBookManager.28"))); //$NON-NLS-1$ //$NON-NLS-2$
		int option = fc.showOpenDialog(frame);
		if (option == JFileChooser.APPROVE_OPTION) {
			File f = fc.getSelectedFile();
			
			// ask for import options
			importOptionsDialog.setVisible(true);
			
			String dateFormat = importOptionsDialog.getChoosedDateFormat();
			if (dateFormat == null)	return;		// user just selected cancel
			String insertPosition = importOptionsDialog.getChoosedInsertPosition();
			
			// try opening file
			try {
				List<LogEntry> impentries = ImportLogBook.importLogBook(f, dateFormat);
				
				//
				// insert at correct position
				//
				if ("Begin".equals(insertPosition)) currentLogBook.addAll(0, impentries); //$NON-NLS-1$
				else currentLogBook.addAll(impentries);
				
				// 
				// alert table model that we have new data in the logbook
				//
				tableModel.fireTableRowsInserted(currentLogBook.size() - impentries.size() - 1, currentLogBook.size() - 1);
				
				JOptionPane.showMessageDialog(frame, 
						MessageFormat.format(Messages.getString("LogBookManager.29"), impentries.size()), //$NON-NLS-1$
						"Import", JOptionPane.INFORMATION_MESSAGE); //$NON-NLS-1$
			} catch (Exception e) {
				JOptionPane.showMessageDialog(frame, 
						MessageFormat.format(Messages.getString("LogBookManager.30"), e.getLocalizedMessage()), //$NON-NLS-1$
						"Error", JOptionPane.ERROR_MESSAGE); //$NON-NLS-1$
			}
			
		}
	}
	
		
	
	/**
	 * Called by GUI on "export" menu choice
	 *
	 */
	private void exportAction() {
		exportDialog.setPrintDialog(false);
		exportDialog.setVisible(true);
	}
	
	private void printAction() {
		exportDialog.setPrintDialog(true);
		exportDialog.setVisible(true);
	}
	

	/**
	 * Show the main window
	 */
	public void show() {
		frame.setVisible(true);
	}
	
	
	/**
	 * Assumes table is contained in a JScrollPane. Scrolls the
	 * cell (rowIndex, vColIndex) so that it is visible at the center of viewport.
	 * 
	 * From the java developer almanac, 1.4
	 */
    private void scrollToCenter(JTable table, int rowIndex, int vColIndex) {
        if (!(table.getParent() instanceof JViewport)) {
            return;
        }
        JViewport viewport = (JViewport)table.getParent();
    
        // This rectangle is relative to the table where the
        // northwest corner of cell (0,0) is always (0,0).
        Rectangle rect = table.getCellRect(rowIndex, vColIndex, true);
    
        // The location of the view relative to the table
        Rectangle viewRect = viewport.getViewRect();
    
        // Translate the cell location so that it is relative
        // to the view, assuming the northwest corner of the
        // view is (0,0).
        rect.setLocation(rect.x-viewRect.x, rect.y-viewRect.y);
    
        // Calculate location of rect if it were at the center of view
        int centerX = (viewRect.width-rect.width)/2;
        int centerY = (viewRect.height-rect.height)/2;
    
        // Fake the location of the cell so that scrollRectToVisible
        // will move the cell to the center
        if (rect.x < centerX) {
            centerX = -centerX;
        }
        if (rect.y < centerY) {
            centerY = -centerY;
        }
        rect.translate(centerX, centerY);
    
        // Scroll the area into view.
        viewport.scrollRectToVisible(rect);
    }
    
	/**
	 * entry point
	 * @param args
	 */
	public static void main(String[] args) {
		LookAndFeel.setPlatformLookAndFeel();
		
		// construct application
		LogBookManager lbm = new LogBookManager();
		lbm.show();
		
		// find a logbook
		File logBookFile = LogBook.findLogBook();
		if (logBookFile != null) lbm.openLogBook(logBookFile);
		
		
	}
}

class ExtensionFileFilter extends FileFilter {
	private String ext;
	private String desc;
	
	ExtensionFileFilter(String ext, String desc) {
		this.ext = ext;
		this.desc = desc;
	}
	
	@Override
	public boolean accept(File f) {
		if (f.isDirectory()) return true;
		if (f.getName().toLowerCase().endsWith("." + ext)) return true; //$NON-NLS-1$
		return false;
	}
	

	@Override
	public String getDescription() {
		return desc;
	}
	
	public String getExtension() {
		return ext;
	}
}


/**
 * A renderer of floating point numbers with rounding
 * @author lc
 *
 */
@SuppressWarnings("serial") //$NON-NLS-1$
class DurationRenderer extends DefaultTableCellRenderer {
	private FloatDurationFormatter fmt;

	DurationRenderer() {
	    super();
		this.fmt = new FloatDurationFormatter();
	    setHorizontalAlignment(JLabel.RIGHT);
	}
	
	public Component getTableCellRendererComponent(JTable table, 
			Object value, 
			boolean isSelected, boolean hasFocus, 
			int row, int column) {
		
		if (isSelected) {
		   super.setForeground(table.getSelectionForeground());
		   super.setBackground(table.getSelectionBackground());
		} else {
		    super.setForeground(table.getForeground());
		    super.setBackground(table.getBackground());
		}

		setFont(table.getFont());
		try {
			setText(fmt.valueToString(value));
		} catch (ParseException e) {
			setText("0"); //$NON-NLS-1$
		}
		return this;
	}
}

/**
 * A renderer of date with fixed format
 * Caveat: is not internationalizable. I should use default date formatters
 * instead (which are localized) but they are not esthetic
 * @author lc
 *
 */
@SuppressWarnings("serial") //$NON-NLS-1$
class SimpleDateRenderer extends DefaultTableCellRenderer  {
	private DateFormat formatter;
	
	SimpleDateRenderer(DateFormat formatter) {
	    super();
	    setHorizontalAlignment(JLabel.LEFT);
	    this.formatter = formatter;
	}
	
	public SimpleDateRenderer(String format) {
		this(new SimpleDateFormat(format));
	}
	
	public Component getTableCellRendererComponent(JTable table, 
			Object value, 
			boolean isSelected, boolean hasFocus, 
			int row, int column) {
		
		if (isSelected) {
		   super.setForeground(table.getSelectionForeground());
		   super.setBackground(table.getSelectionBackground());
		} else {
		    super.setForeground(table.getForeground());
		    super.setBackground(table.getBackground());
		}
		setFont(table.getFont());
		setText(formatter.format((Date)value));
		return this;
	}
	
}

/**
 * The table model
 * @author lc
 *
 */
@SuppressWarnings("serial") //$NON-NLS-1$
class LogBookTableModel extends AbstractTableModel {
	private LogBook currentBook;
	private static final String[] COLUMN_NAMES = {
		Messages.getString("LogBookManager.32"), //$NON-NLS-1$
		Messages.getString("LogBookManager.33"), //$NON-NLS-1$
		Messages.getString("LogBookManager.34"), //$NON-NLS-1$
		Messages.getString("LogBookManager.35"), //$NON-NLS-1$
		Messages.getString("LogBookManager.36"), //$NON-NLS-1$
		Messages.getString("LogBookManager.37"), //$NON-NLS-1$
		Messages.getString("LogBookManager.38"), //$NON-NLS-1$
		Messages.getString("LogBookManager.39"), //$NON-NLS-1$
		Messages.getString("LogBookManager.40"), //$NON-NLS-1$
		Messages.getString("LogBookManager.41") //$NON-NLS-1$
	};

	private static final Class[] COLUMN_CLASSES = {
		Date.class, 
		Date.class,
		String.class, 
		String.class, 
		String.class, 
		String.class,
		Float.class, 
		Float.class, 
		Float.class,
		String.class
	};

	public int getColumnCount() {
		return COLUMN_NAMES.length;
	}


	public LogEntry getRow(int row) {
		return currentBook.getEntries().get(row);
	}

	
	/**
	 * Add a new row and return its index
	 * @return
	 */
	public int addRow() {
		currentBook.addNewEntry();
		fireTableRowsInserted(currentBook.size() - 1, currentBook.size() - 1);	
		return currentBook.size() - 1;
	}

	public void delEntry(int idx) {
		currentBook.getEntries().remove(idx);
		fireTableRowsDeleted(idx, idx);
	}

	public int getRowCount() {
		if (currentBook != null) return currentBook.size();
		return 0;
	}
	
	public Class<?> getColumnClass(int columnIndex) {
		return COLUMN_CLASSES[columnIndex];
	}
	
	public Object getValueAt(int rowIndex, int columnIndex) {
		if (currentBook == null) return null;
		LogEntry le = currentBook.getEntries().get(rowIndex);
		if (columnIndex == 0 || columnIndex == 1) {
			return le.getDate();
		} else if (columnIndex == 2) {
			return le.getFrom();
		} else if (columnIndex == 3) {
			return le.getTo();
		} else if (columnIndex == 4) {
			return le.getPlaneReg();
		} else if (columnIndex == 5) {
			return le.getPlaneDesc();
		} else if (columnIndex == 6) {
			return le.getTotalTime();
		} else if (columnIndex == 7) {
			return le.getNightTime();
		} else if (columnIndex == 8) {
			return le.getInstrumentTime();
		} else if (columnIndex == 9) {
			return le.getDescription();
		}
		return null;
	}
	
	@Override
	public boolean isCellEditable(int rowIndex, int columnIndex) {
		return false;
	}
	
	@Override
	public String getColumnName(int column) {
		return COLUMN_NAMES[column];
	}
	
	public void setLogBook(LogBook currentLogBook) {
		this.currentBook = currentLogBook;
		fireTableDataChanged();
	}
}


/**
 * @author lc
 *
 */
@SuppressWarnings("serial") //$NON-NLS-1$
class ImportOptionsDialog extends JDialog {

	
	private static String[] datePatterns;
	
	private static void initDatePatterns() {
		DateFormat df = SimpleDateFormat.getDateInstance(SimpleDateFormat.SHORT);
		int i = 0;
		if (df instanceof SimpleDateFormat) {
			SimpleDateFormat sdf = (SimpleDateFormat) df;
			datePatterns = new String[5];
			datePatterns[i++] = sdf.toLocalizedPattern() + " " + Messages.getString("LogBookManager.43"); //$NON-NLS-1$ //$NON-NLS-2$
		} else {
			datePatterns  = new String[4];
		}
		datePatterns[i++] = "d/M/yyyy"; //$NON-NLS-1$
		datePatterns[i++] = "M/d/yyyy"; //$NON-NLS-1$
		datePatterns[i++] = "d/M/yy"; //$NON-NLS-1$
		datePatterns[i++] = "M/d/yy"; //$NON-NLS-1$
	}
	
	private String choosedDateFormat;
	private String choosedInsertPosition;
	private JComboBox dateFormatComboBox;
	private ButtonGroup insertPositionGroup;

	ImportOptionsDialog(JFrame parent) {
		super(parent, true);
		setTitle(Messages.getString("LogBookManager.48")); //$NON-NLS-1$
		setSize(450, 210);
		setResizable(true);
		
		JPanel mainPanel = new JPanel(new GridLayout(3,2));
		mainPanel.setBorder(BorderFactory.createTitledBorder("Import options")); //$NON-NLS-1$
		
		if (datePatterns == null) initDatePatterns();
		
		dateFormatComboBox = new JComboBox(datePatterns);
		
		
		mainPanel.add(new JLabel("Date format")); //$NON-NLS-1$
		mainPanel.add(dateFormatComboBox);
		mainPanel.add(new JLabel("Insert position")); //$NON-NLS-1$
		insertPositionGroup = new ButtonGroup();
		JRadioButton but = new JRadioButton("Begin"); //$NON-NLS-1$
		but.setActionCommand("Begin"); //$NON-NLS-1$
		insertPositionGroup.add(but);
		mainPanel.add(but);
		mainPanel.add(new JLabel("")); //$NON-NLS-1$
		but = new JRadioButton("End"); //$NON-NLS-1$
		but.setActionCommand("End"); //$NON-NLS-1$
		but.setSelected(true);
		insertPositionGroup.add(but);
		mainPanel.add(but);
		
		mainPanel.validate();
		
		// Create an option pane with mainpanel into it
		final JOptionPane optionPane = new JOptionPane(mainPanel, JOptionPane.PLAIN_MESSAGE, JOptionPane.OK_CANCEL_OPTION);
		
		// discard window if closed
		setDefaultCloseOperation(HIDE_ON_CLOSE);
		
		optionPane.addPropertyChangeListener(new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent evt) {
		        String prop = evt.getPropertyName();
		        if (isVisible()
		         && (evt.getSource() == optionPane)
		         && (JOptionPane.VALUE_PROPERTY.equals(prop) ||
		             JOptionPane.INPUT_VALUE_PROPERTY.equals(prop))) {
		            Object value = optionPane.getValue();
		            if (value == JOptionPane.UNINITIALIZED_VALUE) {
		                //ignore reset
		                return;
		            }
		            if (value.equals(JOptionPane.OK_OPTION)) {
		            	choosedDateFormat = (String) dateFormatComboBox.getSelectedItem();
		            	choosedInsertPosition = insertPositionGroup.getSelection().getActionCommand();
		            }
		            
		            // if cancel, set choosen to null
		            if (value.equals(JOptionPane.CANCEL_OPTION)) {
		            	choosedDateFormat = null;
		            	choosedInsertPosition = null;
		            	
		            }
		            // reset it
		            optionPane.setValue(JOptionPane.UNINITIALIZED_VALUE);
		            // close
		            ImportOptionsDialog.this.setVisible(false);
		        }
		        }});
		setContentPane(optionPane);

	}
	
	/**
	 * Returns the usable date format (without the origin comment)
	 * @return
	 */
	public String getChoosedDateFormat() {
		if (choosedDateFormat == null) return null;
		
		int ip = choosedDateFormat.indexOf(' ');
		if (ip != -1) choosedDateFormat = choosedDateFormat.substring(0, ip);
		return choosedDateFormat;
	}
	
	/**
	 * Returns the insert position, either <i>Begin</i> or <i>End</i>
	 * @return
	 */
	public String getChoosedInsertPosition() {
		return choosedInsertPosition;
	}
	
}

