package flightsim.logbook.gui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.EventObject;
import java.util.List;
import java.util.Locale;

import javax.swing.AbstractCellEditor;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFormattedTextField;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.SpinnerNumberModel;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableCellEditor;
import javax.swing.text.AbstractDocument;
import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.DocumentFilter;


import flightsim.logbook.LogEntry;

/**
 * TODO: Some Tooltips
 * @author Laurent
 * $Id: LogEntryEditor.java,v 1.1 2007-02-03 09:26:52 lc Exp $
 */
@SuppressWarnings("serial") //$NON-NLS-1$
class LogEntryEditor extends JDialog {

	private LogEntry logEntry;
	
	private final JFormattedTextField hourEditor;
	private final JFormattedTextField dateEditor;
	
	private JTextField fromAirportEditor;
	private JTextField toAirportEditor;

	private JTextField planeRegEditor;
	private JTextField planeDescEditor;
	private JComboBox planeTypeEditor;
	private JCheckBox planeMultimotor;
	
	private JTextField flightDescEditor;
	
	private JFormattedTextField timeTotalEditor;
	private JFormattedTextField timeNightEditor;
	private JFormattedTextField timeInstrumentEditor;

	private JTable landingTable;

	private JOptionPane optionPane;
	private boolean modified;					
	
	private static final DateFormat DATE_FORMAT = DateFormat.getDateInstance(DateFormat.SHORT, Locale.getDefault());
	private static final DateFormat TIME_FORMAT = new SimpleDateFormat("HH:mm:ss");		// skip locale //$NON-NLS-1$
	
	private void addTitled(JComponent what, JComponent to, String title, GridBagConstraints c) {
		JPanel tmp1 = new JPanel(new BorderLayout());
		tmp1.setBorder(BorderFactory.createTitledBorder(title));
		tmp1.add(what, BorderLayout.NORTH);
		to.add(tmp1, c);
	}
	
	public LogEntryEditor(JFrame parent) {
		super(parent, true);
		
		setTitle(Messages.getString("LogEntryEditor.0")); //$NON-NLS-1$
		setSize(650, 400);
		setResizable(true);
		setLayout(new BorderLayout());
		
		//
		// build widgets
		//
		
		JPanel mainPanel = new JPanel(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		
		c.weighty = 0.5;
		// hour/date panel
		hourEditor = new JFormattedTextField(TIME_FORMAT);
		dateEditor = new JFormattedTextField(DATE_FORMAT);
		hourEditor.setEditable(true);
		dateEditor.setEditable(true);
		JPanel hourDatePanel = new JPanel(new GridLayout(2,2));
		hourDatePanel.add(new JLabel(Messages.getString("LogEntryEditor.1"))); //$NON-NLS-1$
		hourDatePanel.add(hourEditor);
		hourDatePanel.add(new JLabel(Messages.getString("LogEntryEditor.2"))); //$NON-NLS-1$
		hourDatePanel.add(dateEditor);
		c.gridx = 0; c.gridy = 0; c.gridwidth = 1; c.gridheight = 1; c.weightx = 0.2; c.anchor = GridBagConstraints.FIRST_LINE_START; c.fill = GridBagConstraints.BOTH;
		addTitled(hourDatePanel, mainPanel, Messages.getString("LogEntryEditor.3"), c);  //$NON-NLS-1$
		
		// from/to panel
		fromAirportEditor = new JTextField("KMMM"); //$NON-NLS-1$
		toAirportEditor = new JTextField("KMMM"); //$NON-NLS-1$
		((AbstractDocument)toAirportEditor.getDocument()).setDocumentFilter(new DocumentSizeFilter(4));
		((AbstractDocument)fromAirportEditor.getDocument()).setDocumentFilter(new DocumentSizeFilter(4));
		JPanel fromToPanel = new JPanel(new GridLayout(2,2));
		fromToPanel.add(new JLabel(Messages.getString("LogEntryEditor.4"))); //$NON-NLS-1$
		fromToPanel.add(fromAirportEditor);
		fromToPanel.add(new JLabel(Messages.getString("LogEntryEditor.5")));  //$NON-NLS-1$
		fromToPanel.add(toAirportEditor);
		c.gridx = 1; c.gridy = 0; c.gridwidth = 1; c.gridheight = 1; c.weightx = 0.2; c.anchor = GridBagConstraints.PAGE_START;c.fill = GridBagConstraints.BOTH;
		addTitled(fromToPanel, mainPanel, Messages.getString("LogEntryEditor.6"), c); //$NON-NLS-1$
		
		// time editors
		timeTotalEditor = new JFormattedTextField(new FloatDurationFormatter());
		timeTotalEditor.setFocusLostBehavior(JFormattedTextField.COMMIT_OR_REVERT);
		timeNightEditor = new JFormattedTextField(new FloatDurationFormatter());
		timeNightEditor.setFocusLostBehavior(JFormattedTextField.COMMIT_OR_REVERT);
		timeInstrumentEditor = new JFormattedTextField(new FloatDurationFormatter());
		timeInstrumentEditor.setFocusLostBehavior(JFormattedTextField.COMMIT_OR_REVERT);
		
		PropertyChangeListener durationsPl = new DurationsCoherencyAdapter();
		timeTotalEditor.addPropertyChangeListener("value", durationsPl); //$NON-NLS-1$
		timeNightEditor.addPropertyChangeListener("value", durationsPl); //$NON-NLS-1$
		timeInstrumentEditor.addPropertyChangeListener("value", durationsPl); //$NON-NLS-1$
		JPanel timesPanel = new JPanel(new GridLayout(3,2));
		timesPanel.add(new JLabel(Messages.getString("LogEntryEditor.7"))); //$NON-NLS-1$
		timesPanel.add(timeTotalEditor);
		timesPanel.add(new JLabel(Messages.getString("LogEntryEditor.8")));  //$NON-NLS-1$
		timesPanel.add(timeNightEditor);
		timesPanel.add(new JLabel(Messages.getString("LogEntryEditor.9")));  //$NON-NLS-1$
		timesPanel.add(timeInstrumentEditor);
		c.gridx = 2; c.gridy = 0; c.gridwidth = 1; c.gridheight = 1; c.weightx = 0.2; c.anchor = GridBagConstraints.PAGE_START;c.fill = GridBagConstraints.BOTH;
		addTitled(timesPanel, mainPanel,Messages.getString("LogEntryEditor.10"), c);  //$NON-NLS-1$

		// plane reg/desc
		planeRegEditor = new JTextField("F-01234567"); //$NON-NLS-1$
		planeDescEditor = new JTextField("Description nulle"); //$NON-NLS-1$
		planeTypeEditor = new JComboBox(LogEntry.AC_TYPES_DESCRIPTION);
		planeMultimotor = new JCheckBox();
		JPanel planePanel = new JPanel(new GridLayout(4,2));
		planePanel.add(new JLabel(Messages.getString("LogEntryEditor.11")));  //$NON-NLS-1$
		planePanel.add(planeRegEditor);
		planePanel.add(new JLabel(Messages.getString("LogEntryEditor.12")));  //$NON-NLS-1$
		planePanel.add(planeDescEditor);
		planePanel.add(new JLabel(Messages.getString("LogEntryEditor.13"))); //$NON-NLS-1$
		planePanel.add(planeTypeEditor);
		planePanel.add(new JLabel(Messages.getString("LogEntryEditor.14")));  //$NON-NLS-1$
		planePanel.add(planeMultimotor);
		c.gridx = 0; c.gridy = 1; c.gridwidth = 3; c.gridheight = 1; c.anchor = GridBagConstraints.LINE_START;c.fill = GridBagConstraints.BOTH;
		addTitled(planePanel, mainPanel, Messages.getString("LogEntryEditor.15"), c);  //$NON-NLS-1$
		
		// landings panel
		landingTable = new JTable(new LandingTableModel());
		landingTable.setSelectionMode( ListSelectionModel.SINGLE_SELECTION);
		
		// custom editors
		landingTable.getColumnModel().getColumn(0).setCellEditor(new LimitedWidthCellEditor(4));
		landingTable.getColumnModel().getColumn(1).setCellEditor(new NumberSpinnerCellEditor(1, 1, 255, 1));
	
		// taller rows
		landingTable.setRowHeight(landingTable.getRowHeight()+ 5);
		
		// panel containing "Add/Del" landing buttons and table
		JPanel landingPanel = new JPanel(new BorderLayout());
		landingPanel.setPreferredSize(new Dimension(50,100));
		landingPanel.add(new JScrollPane(landingTable), BorderLayout.CENTER);
		
		JPanel ldgButtonsPanel = new JPanel(new FlowLayout());
		JButton addLdgButton = new JButton(Messages.getString("LogEntryEditor.16"));  //$NON-NLS-1$
		addLdgButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				((LandingTableModel)landingTable.getModel()).addRow();
			}});
		ldgButtonsPanel.add(addLdgButton);
		
		JButton delLdgButton = new JButton(Messages.getString("LogEntryEditor.17"));  //$NON-NLS-1$
		delLdgButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				((LandingTableModel)landingTable.getModel()).delRow(landingTable.getSelectedRow());
			}});
		ldgButtonsPanel.add(delLdgButton);
		landingPanel.add(ldgButtonsPanel, BorderLayout.SOUTH);
		
		c.gridx = 3; c.gridy = 0; c.gridwidth = 1; c.gridheight = 2; c.weightx = 0.9; c.anchor = GridBagConstraints.FIRST_LINE_END; c.fill = GridBagConstraints.BOTH;
		JPanel tmp1 = new JPanel(new BorderLayout());
		tmp1.setBorder(BorderFactory.createTitledBorder(Messages.getString("LogEntryEditor.18")));  //$NON-NLS-1$
		tmp1.add(new JScrollPane(landingPanel), BorderLayout.CENTER);
		mainPanel.add(tmp1, c);
//		addTitled(landingPanel, mainPanel, "Landing", c);
		
		// desc
		flightDescEditor = new JTextField("Description a la con"); //$NON-NLS-1$
		JPanel flightDescPanel = new JPanel(new GridLayout(2,1));
		flightDescPanel.add(new JLabel(Messages.getString("LogEntryEditor.19")));  //$NON-NLS-1$
		flightDescPanel.add(flightDescEditor);
		c.gridx = 0; c.gridy = 2; c.gridwidth = 4; c.gridheight = 1; c.anchor = GridBagConstraints.PAGE_END; c.fill = GridBagConstraints.BOTH;
		addTitled(flightDescPanel, mainPanel, Messages.getString("LogEntryEditor.20"), c);  //$NON-NLS-1$
		
		mainPanel.validate();
		
		// Create an option pane with mainpanel into it
		optionPane = new JOptionPane(mainPanel, JOptionPane.PLAIN_MESSAGE, JOptionPane.OK_CANCEL_OPTION);
		
		// discard window if closed
		setDefaultCloseOperation(HIDE_ON_CLOSE);
		
		// add a change listener that will modify logentry when "OK"
		// is clicked. The chosen button of the optionPane is reflected
		// in the "value" property of it. 
		optionPane.addPropertyChangeListener(new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent evt) {
		        String prop = evt.getPropertyName();
		        if (isVisible()
		         && (evt.getSource() == optionPane)
		         && (JOptionPane.VALUE_PROPERTY.equals(prop) ||
		             JOptionPane.INPUT_VALUE_PROPERTY.equals(prop))) {
		            Object value = optionPane.getValue();
		            if (value == JOptionPane.UNINITIALIZED_VALUE) {
		                //ignore reset
		                return;
		            }
		            if (value.equals(JOptionPane.OK_OPTION)) {
		            	modified = true;
		            	updateLogEntry();
		            }
		            // reset it
		            optionPane.setValue(
		                    JOptionPane.UNINITIALIZED_VALUE);
		            // close
		            LogEntryEditor.this.setVisible(false);
		        }
		        }});
		setContentPane(optionPane);
	}
	
	public boolean isModified() {
		return modified;
	}
	
	public void setLogEntry(LogEntry le) {
		this.logEntry = le;
		
		modified = false;		// no modification saved
		synchronized (le) {
			// refresh widgets
			dateEditor.setValue(le.getDate());
			hourEditor.setValue(le.getDate());
			flightDescEditor.setText(le.getDescription());
			fromAirportEditor.setText(le.getFrom());
			toAirportEditor.setText(le.getTo());
			planeDescEditor.setText(le.getPlaneDesc());
			planeRegEditor.setText(le.getPlaneReg());
			timeTotalEditor.setValue(new Float(le.getTotalTime()));
			timeInstrumentEditor.setValue(new Float(le.getInstrumentTime()));
			timeNightEditor.setValue(new Float(le.getNightTime()));
			if (le.getAircraftType() < LogEntry.AC_TYPES_DESCRIPTION.length) 
				planeTypeEditor.setSelectedIndex(le.getAircraftType() );
			else
				planeTypeEditor.setSelectedIndex(0);
			planeMultimotor.setSelected(le.isMultimotor());
			
			// update landing tables
			((LandingTableModel) landingTable.getModel()).setLogEntry(le);
		}
	}
	
	public void updateLogEntry() {
		if (logEntry == null) return;
		if (dateEditor.getValue() == null) return;
		if (hourEditor.getValue() == null) return;
		
		synchronized (logEntry) {
			Calendar cal = Calendar.getInstance();
			cal.setTime((Date) dateEditor.getValue());
			Calendar cal2 = Calendar.getInstance();
			cal2.setTime((Date) hourEditor.getValue());
			cal2.set(cal.get(Calendar.YEAR),
					cal.get(Calendar.MONTH),
					cal.get(Calendar.DAY_OF_MONTH));
			logEntry.setDate(cal2.getTime());
			logEntry.setDescription(flightDescEditor.getText());
			logEntry.setPlaneDesc(planeDescEditor.getText());
			logEntry.setPlaneReg(planeRegEditor.getText());
			logEntry.setTotalTime((Float)timeTotalEditor.getValue());
			logEntry.setNightTime((Float)timeNightEditor.getValue());
			logEntry.setInstrumentTime((Float)timeInstrumentEditor.getValue());
			logEntry.setFrom(fromAirportEditor.getText());
			logEntry.setTo(toAirportEditor.getText());
			// update only if a 'known' value is choosen
			int acTypeSelectedIndex = planeTypeEditor.getSelectedIndex();
			if (acTypeSelectedIndex != 0) logEntry.setAircraftType(acTypeSelectedIndex);
			logEntry.setMultimotor(planeMultimotor.isSelected());
			
			// update landing tables
			LandingTableModel ltbm = (LandingTableModel) landingTable.getModel();
			ltbm.updateLogEntry(logEntry);
		}
	}
	
	
	@SuppressWarnings("unused") //$NON-NLS-1$
	private class DurationsCoherencyAdapter implements PropertyChangeListener {
		public void propertyChange(PropertyChangeEvent evt) {
			Object src = evt.getSource();
			Float val = (Float) timeTotalEditor.getValue();
			Float vali = (Float) timeInstrumentEditor.getValue();
			Float valn = (Float) timeNightEditor.getValue();
			
			// no value yet
			if (vali == null || val == null || valn == null) return;
			
			if (src == timeTotalEditor) {
				// if downing, put the minimum
				if (vali > val)
					timeInstrumentEditor.setValue(val);
				if (valn > val)
					timeNightEditor.setValue(val);
			}
			if (src == timeNightEditor) {
				// increase total if needed
				if (valn > val) timeTotalEditor.setValue(valn);
			}
			if (src == timeInstrumentEditor) {
				// increase if needed
				if (vali > val) timeTotalEditor.setValue(vali);
			}
		}
		
	}

	
	@SuppressWarnings("serial") //$NON-NLS-1$
	private class LandingTableModel extends AbstractTableModel {
		private List<String> landingAirports = new ArrayList<String>();
		private List<Integer> landingTimes = new ArrayList<Integer>();
		
		public int getColumnCount() {
			return 2;
		}

		public void delRow(int selectedRow) {
			if (selectedRow > 0 && selectedRow < landingAirports.size()) {
				synchronized (this) {
					landingAirports.remove(selectedRow);
					landingTimes.remove(selectedRow);
				}
				fireTableRowsDeleted(selectedRow, selectedRow);
			} else {
				
			}
		}

		public void addRow() {
			synchronized (this) {
				landingAirports.add(Messages.getString("LogEntryEditor.21")); //$NON-NLS-1$
				landingTimes.add(1);
			}
			fireTableRowsInserted(landingAirports.size(), landingAirports.size());
			requestFocus();
		}

		/**
		 * Copy data from logentry
		 * @param le
		 */
		public void setLogEntry(LogEntry le) {
			synchronized (this) {
				synchronized (logEntry) {
					landingAirports.clear();
					landingAirports.addAll(le.getLandingAirports());
					landingTimes.clear();
					landingTimes.addAll(le.getLandingTimes());
				}
			}
			fireTableDataChanged();
		}

		
		/**
		 * Copy data to logentry
		 * @param logEntry
		 */
		public void updateLogEntry(LogEntry logEntry) {
			synchronized (this) {
				synchronized (logEntry) {
					logEntry.clearLandingTable();
					for (int i = 0; i < landingAirports.size(); i++) {
						logEntry.addLanding(landingAirports.get(i), landingTimes.get(i));
					}
				}
			}
		}

		public int getRowCount() {
			return landingAirports.size();
		}

		public Object getValueAt(int rowIndex, int columnIndex) {
			if (columnIndex == 0) {
				return landingAirports.get(rowIndex);
			} else {
				return landingTimes.get(rowIndex);
			}
		}
		
		public String getColumnName(int column) {
			if (column == 0) return Messages.getString("LogEntryEditor.22");  //$NON-NLS-1$
			else return Messages.getString("LogEntryEditor.23"); //$NON-NLS-1$
		}
		
		public Class<?> getColumnClass(int columnIndex) {
			if (columnIndex == 0) return String.class;
			else return Integer.class;
		}
		
		public boolean isCellEditable(int rowIndex, int columnIndex) {
			return true;
		}
		
		public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
			if (columnIndex == 0) {
				landingAirports.set(rowIndex, (String) aValue);
			} else if (columnIndex == 1) {
				landingTimes.set(rowIndex, (Integer) aValue);
			}
			fireTableRowsUpdated(rowIndex, rowIndex);
		}
		
	}
}



@SuppressWarnings("serial") //$NON-NLS-1$
class LimitedWidthCellEditor extends AbstractCellEditor implements TableCellEditor {

	private JTextField field = new JTextField();
	LimitedWidthCellEditor(int width) {
		((AbstractDocument)field.getDocument()).setDocumentFilter(new DocumentSizeFilter(width));
	}
	
	public Component getTableCellEditorComponent(JTable table, Object value, boolean isSelected, int row, int column) {
		field.setText((String) value);
		return field;
	}

	public Object getCellEditorValue() {
		return field.getText();
	}
	
}

class DocumentSizeFilter extends DocumentFilter {
    int maxCharacters;

    public DocumentSizeFilter(int maxChars) {
        maxCharacters = maxChars;
    }

    public void insertString(FilterBypass fb, int offs,
                             String str, AttributeSet a)
        throws BadLocationException {

        if ((fb.getDocument().getLength() + str.length()) <= maxCharacters)
            super.insertString(fb, offs, str, a);
        else
            Toolkit.getDefaultToolkit().beep();
    }
    
    public void replace(FilterBypass fb, int offs,
                        int length, 
                        String str, AttributeSet a)
        throws BadLocationException {
        if ((fb.getDocument().getLength() + str.length()
             - length) <= maxCharacters)
            super.replace(fb, offs, length, str, a);
        else
            Toolkit.getDefaultToolkit().beep();
    }

}

@SuppressWarnings("serial") //$NON-NLS-1$
class NumberSpinnerCellEditor extends AbstractCellEditor implements TableCellEditor  {
	private JSpinner spinner;
	
	NumberSpinnerCellEditor(int val, int min, int max, int step) {
		spinner = new JSpinner(new SpinnerNumberModel(val, min, max, step));
	}
	
	public Component getTableCellEditorComponent(JTable table, Object value, boolean isSelected, int row, int column) {
		spinner.setValue(value);
		return spinner;
	}
	
	@Override
	public boolean isCellEditable(EventObject e) {
		if (e instanceof MouseEvent) {
			MouseEvent me = (MouseEvent) e;
			return me.getClickCount() >= 2;
		}
		return true;
	}
	
	public Object getCellEditorValue() {
		return spinner.getValue();
	}
	
}
