package flightsim.logbook.gui.exportfilters;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.text.DateFormat;
import java.util.*;

import flightsim.logbook.LogBook;
import flightsim.logbook.LogEntry;


public abstract class Exporter {

	// list of all fields
	private static List<ExportField> exfields;
	
	static {
		exfields = new LinkedList<ExportField>();
		// build all export fields
		exfields.add(new StringField(Messages.getString("Exporter.0"), "From airport ICAO"){ //$NON-NLS-1$ //$NON-NLS-2$
			public String format(LogEntry value) {
				return value.getFrom();
			}});
		exfields.add(new StringField(Messages.getString("Exporter.2"), "To airport ICAO"){ //$NON-NLS-1$ //$NON-NLS-2$
			public String format(LogEntry value) {
				return value.getTo();
			}});
		exfields.add(new StringField(Messages.getString("Exporter.4"), "From/To airport ICAO"){ //$NON-NLS-1$ //$NON-NLS-2$
			public String format(LogEntry value) {
				return value.getFrom() + "->" + value.getTo(); //$NON-NLS-1$
			}});
		exfields.add(new StringField(Messages.getString("Exporter.7"), "Plane registration"){ //$NON-NLS-1$ //$NON-NLS-2$
			public String format(LogEntry value) {
				return value.getPlaneReg();
			}});
		exfields.add(new StringField(Messages.getString("Exporter.9"), "Plane description"){ //$NON-NLS-1$ //$NON-NLS-2$
			public String format(LogEntry value) {
				return value.getPlaneDesc();
			}});
		exfields.add(new StringField(Messages.getString("Exporter.11"), "Flight description"){ //$NON-NLS-1$ //$NON-NLS-2$
			public String format(LogEntry value) {
				return value.getDescription();
			}});
		exfields.add(new ShortDurationField(Messages.getString("Exporter.13"), "Flight time (short)"){ //$NON-NLS-1$ //$NON-NLS-2$
			protected float getValue(LogEntry le) {
				return le.getTotalTime();
			}});
		exfields.add(new ShortDurationField(Messages.getString("Exporter.15"), "Night time (short)"){ //$NON-NLS-1$ //$NON-NLS-2$
			protected float getValue(LogEntry le) {
				return le.getNightTime();
			}});
		exfields.add(new ShortDurationField(Messages.getString("Exporter.17"), "Instrument time (short)"){ //$NON-NLS-1$ //$NON-NLS-2$
			protected float getValue(LogEntry le) {
				return le.getInstrumentTime();
			}});
		exfields.add(new ShortDurationField(Messages.getString("Exporter.19"), "Day time (short)"){ //$NON-NLS-1$ //$NON-NLS-2$
			protected float getValue(LogEntry le) {
				return le.getDayTime();
			}});
		exfields.add(new LongDurationField(Messages.getString("Exporter.21"), "Flight time (medium)"){ //$NON-NLS-1$ //$NON-NLS-2$
			protected float getValue(LogEntry le) {
				return le.getTotalTime();
			}});
		exfields.add(new LongDurationField(Messages.getString("Exporter.23"), "Night time (medium)"){ //$NON-NLS-1$ //$NON-NLS-2$
			protected float getValue(LogEntry le) {
				return le.getNightTime();
			}});
		exfields.add(new LongDurationField(Messages.getString("Exporter.25"), "Instrument time (medium)"){ //$NON-NLS-1$ //$NON-NLS-2$
			protected float getValue(LogEntry le) {
				return le.getInstrumentTime();
			}});
		exfields.add(new LongDurationField(Messages.getString("Exporter.27"), "Day time (medium)"){ //$NON-NLS-1$ //$NON-NLS-2$
			protected float getValue(LogEntry le) {
				return le.getDayTime();
			}});
		exfields.add(new ExtraLongDurationField(Messages.getString("Exporter.29"), "Flight time (long format)"){ //$NON-NLS-1$ //$NON-NLS-2$
			protected float getValue(LogEntry le) {
				return le.getTotalTime();
			}});
		exfields.add(new ExtraLongDurationField(Messages.getString("Exporter.31"), "Night time (long format)"){ //$NON-NLS-1$ //$NON-NLS-2$
			protected float getValue(LogEntry le) {
				return le.getNightTime();
			}});
		exfields.add(new ExtraLongDurationField(Messages.getString("Exporter.33"), "Instrument time (long format)"){ //$NON-NLS-1$ //$NON-NLS-2$
			protected float getValue(LogEntry le) {
				return le.getInstrumentTime();
			}});
		exfields.add(new ExtraLongDurationField(Messages.getString("Exporter.35"), "Day time (long format)"){ //$NON-NLS-1$ //$NON-NLS-2$
			protected float getValue(LogEntry le) {
				return le.getDayTime();
			}});
		exfields.add(new ExportField(Messages.getString("Exporter.37"), "Number of landings") { //$NON-NLS-1$ //$NON-NLS-2$
			String format(LogEntry value) {
				int t = 1;
				for (int i = 0; i < value.numberLandings(); i++) t+= value.getLandingTimes(i);
				return Integer.toString(t);
			}
		});
		exfields.add(new ExportField(Messages.getString("Exporter.39"), "Number of landings") { //$NON-NLS-1$ //$NON-NLS-2$
			String format(LogEntry value) {
				int t = 1;
				for (int i = 0; i < value.numberLandings(); i++) t+= value.getLandingTimes(i);
				return Integer.toString(t);
			}
		});
		exfields.add(new AirportLandingsField(Messages.getString("Exporter.41"), "Landings airport (space separated)", "")); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		exfields.add(new AirportLandingsField(Messages.getString("Exporter.44"), "Landings airport (newline separated)", "<br/>")); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		exfields.add(new DateTimeField(Messages.getString("Exporter.47"), "Date (long)", DATETIME_SELECTION.DATE, DateFormat.LONG)); //$NON-NLS-1$ //$NON-NLS-2$
		exfields.add(new DateTimeField(Messages.getString("Exporter.49"), "Date (long, force US format)", DATETIME_SELECTION.DATE, DateFormat.LONG, Locale.US)); //$NON-NLS-1$ //$NON-NLS-2$
		exfields.add(new DateTimeField(Messages.getString("Exporter.51"), "Date (medium)", DATETIME_SELECTION.DATE, DateFormat.MEDIUM)); //$NON-NLS-1$ //$NON-NLS-2$
		exfields.add(new DateTimeField(Messages.getString("Exporter.53"), "Date (medium, force US format)", DATETIME_SELECTION.DATE, DateFormat.MEDIUM, Locale.US)); //$NON-NLS-1$ //$NON-NLS-2$
		exfields.add(new DateTimeField(Messages.getString("Exporter.55"), "Date (short)", DATETIME_SELECTION.DATE, DateFormat.SHORT)); //$NON-NLS-1$ //$NON-NLS-2$
		exfields.add(new DateTimeField(Messages.getString("Exporter.57"), "Date (short, force US format)", DATETIME_SELECTION.DATE, DateFormat.SHORT, Locale.US)); //$NON-NLS-1$ //$NON-NLS-2$
		exfields.add(new DateTimeField(Messages.getString("Exporter.59"), "Time (long)", DATETIME_SELECTION.TIME, DateFormat.LONG)); //$NON-NLS-1$ //$NON-NLS-2$
		exfields.add(new DateTimeField(Messages.getString("Exporter.61"), "Time (medium)", DATETIME_SELECTION.TIME, DateFormat.MEDIUM)); //$NON-NLS-1$ //$NON-NLS-2$
		exfields.add(new DateTimeField(Messages.getString("Exporter.63"), "Time (short)", DATETIME_SELECTION.TIME, DateFormat.SHORT)); //$NON-NLS-1$ //$NON-NLS-2$
		exfields.add(new DateTimeField(Messages.getString("Exporter.65"), "Date/Time (long)", DATETIME_SELECTION.DATE_TIME, DateFormat.LONG)); //$NON-NLS-1$ //$NON-NLS-2$
		exfields.add(new DateTimeField(Messages.getString("Exporter.67"), "Date/Time (medium)", DATETIME_SELECTION.DATE_TIME, DateFormat.MEDIUM)); //$NON-NLS-1$ //$NON-NLS-2$
		exfields.add(new DateTimeField(Messages.getString("Exporter.69"), "Date/Time (short)", DATETIME_SELECTION.DATE_TIME, DateFormat.SHORT)); //$NON-NLS-1$ //$NON-NLS-2$
		exfields.add(new ExportField(Messages.getString("Exporter.71"), "Plane type") { //$NON-NLS-1$ //$NON-NLS-2$
			String format(LogEntry value) {
				int type = value.getAircraftType();
				if (type > 0 && type < LogEntry.AC_TYPES_DESCRIPTION.length)
					return LogEntry.AC_TYPES_DESCRIPTION[type];
				return Messages.getString("Exporter.78"); //$NON-NLS-1$
			}});
		exfields.add(new ExportField(Messages.getString("Exporter.74"), "Is aircraft multimotor?"){ //$NON-NLS-1$ //$NON-NLS-2$
			@Override
			String format(LogEntry value) {
				return Boolean.toString(value.isMultimotor());
			}
		});
	}
	
	public static ExportField getExportField(String descName) {
		for (ExportField ef : exfields) {
			if (ef.getDescription().equalsIgnoreCase(descName)) return ef;
		}
		return null;
	}
	
	public static List<ExportField> getExportableFields() {
		return exfields;
	}
	
	public static Exporter getHTMLExporter(List<ExportField> choosed) {
		return new HTMLExporter(choosed);
	}

	public static Exporter getCSVExporter(List<ExportField> choosed) {
		return new CSVExporter(choosed);
	}
	
	public static PrintExporter getPrintExporter(List<ExportField> choosed, LogBook lb) {
		return new PrintExporter(choosed, lb);
	}
	
	
	protected List<ExportField> choosedFields;
	
	protected Exporter(List<ExportField> choosed) {
		this.choosedFields = choosed;
	}
	
	protected abstract void writeFileHeader(LogBook lb, PrintWriter pw);
	protected abstract void writeEntryBegin(LogBook lb, LogEntry le, PrintWriter pw);
	protected abstract void writeEntry(LogBook lb, LogEntry le, ExportField field, PrintWriter pw);
	protected abstract void writeEntryEnd(LogBook lb, LogEntry le, PrintWriter pw);
	protected abstract void writeFileFooter(LogBook lb, PrintWriter pw);
	
	public void writeLogBook(LogBook lb, File outFile) throws IOException {
		PrintWriter pw = new PrintWriter(new FileWriter(outFile));
		writeFileHeader(lb, pw);
		for (LogEntry le : lb) {
			writeEntryBegin(lb, le, pw);
			for (ExportField ef : choosedFields) {
				writeEntry(lb, le, ef, pw);
			}
			writeEntryEnd(lb, le, pw);
		}
		writeFileFooter(lb, pw);
		pw.flush();
		pw.close();
	}

	/**
	 * Test main
	 * @param args
	 */
	public static void main(String[] args) throws Exception {
		LogBook lb = new LogBook("/home/lc/mnt/Logbook.BIN"); //$NON-NLS-1$
		Exporter e = Exporter.getCSVExporter(getExportableFields());
		e.writeLogBook(lb, new File("test.out")); //$NON-NLS-1$
		
	}
}
