package flightsim.logbook.gui.exportfilters;

import java.io.PrintWriter;
import java.util.List;

import flightsim.logbook.LogBook;
import flightsim.logbook.LogEntry;


class HTMLExporter extends Exporter {

	private float totalHours;
	private float totalIFRHours;
	private float totalNightHours;
	
	public HTMLExporter(List<ExportField> choosed) {
		super(choosed);
	}

	@Override
	protected void writeEntry(LogBook lb, LogEntry le, ExportField field, PrintWriter pw) {
		pw.print("<td>"); //$NON-NLS-1$
		pw.print(field.format(le));
		pw.print("</td>"); //$NON-NLS-1$
	}

	@Override
	protected void writeEntryBegin(LogBook lb, LogEntry le, PrintWriter pw) {
		pw.print("<tr>"); //$NON-NLS-1$
		totalHours += le.getTotalTime();
		totalIFRHours += le.getInstrumentTime();
		totalNightHours += le.getNightTime();
	}

	@Override
	protected void writeEntryEnd(LogBook lb, LogEntry le, PrintWriter pw) {
		pw.print("</tr>"); //$NON-NLS-1$
		pw.println();
		pw.flush();
	}

	@Override
	protected void writeFileFooter(LogBook lb, PrintWriter pw) {
		pw.println("</table>"); //$NON-NLS-1$
		pw.println("<hr/>"); //$NON-NLS-1$
		pw.println("<br/>"); //$NON-NLS-1$
		pw.println(Messages.getString("HTMLExporter.7") + ExtraLongDurationField.format(totalHours) +"</i>"); //$NON-NLS-1$ //$NON-NLS-2$
		pw.println(Messages.getString("HTMLExporter.9") + ExtraLongDurationField.format(totalIFRHours) + "</i>"); //$NON-NLS-1$ //$NON-NLS-2$
		pw.println(Messages.getString("HTMLExporter.11") + ExtraLongDurationField.format(totalNightHours) + "</i>"); //$NON-NLS-1$ //$NON-NLS-2$
		pw.println("<br/>"); //$NON-NLS-1$
		pw.println("</body>"); //$NON-NLS-1$
		pw.println("</html>"); //$NON-NLS-1$
		
	}

	@Override
	protected void writeFileHeader(LogBook lb, PrintWriter pw) {
		pw.println("<html>"); //$NON-NLS-1$
		pw.println("<head>"); //$NON-NLS-1$
		pw.println(Messages.getString("HTMLExporter.18")); //$NON-NLS-1$
		pw.println("</head>"); //$NON-NLS-1$
		pw.println("<body bgcolor=\"#fffdc6\">"); //$NON-NLS-1$
		pw.println(Messages.getString("HTMLExporter.21")); //$NON-NLS-1$
		pw.println(Messages.getString("HTMLExporter.22") + lb.getFile() + "</i>"); //$NON-NLS-1$ //$NON-NLS-2$
		pw.println("<hr/>"); //$NON-NLS-1$
		
		pw.println("<table border=\"0\" width=\"100%\">"); //$NON-NLS-1$
		
		// write header
		pw.println("<tr>"); //$NON-NLS-1$
		for (ExportField ef : choosedFields) {
			pw.print("<th>"); //$NON-NLS-1$
			pw.print(ef.getName());
			pw.println("</th>"); //$NON-NLS-1$
		}
		pw.println("</tr>"); //$NON-NLS-1$
	}

}
