package flightsim.logbook.gui.exportfilters;

import java.text.DecimalFormat;
import java.text.NumberFormat;

import flightsim.logbook.LogEntry;


/**
 * Short Format duration is in the form "28:30"
 * Long is "2 days 4:40"
 * Extralong is "2days 4h 40m 31s"
 * @author lc
 *
 */
abstract class LongDurationField extends ExportField {

	protected LongDurationField(String name, String description) {
		super(name, description);
	}

	private static final NumberFormat FMT = new DecimalFormat("00"); //$NON-NLS-1$
	
	@Override
	String format(LogEntry value) {
		float f = getValue(value);
		int nhours = (int) f;
		int nminutes = (int)((f - nhours) * 60.0);
		if (nhours >= 24) {
			int ndays = (int) (f / 24);
			nhours = (int)(f % 24);
			return ndays +Messages.getString("LongDurationField.0") + //$NON-NLS-1$
			((ndays>1) ? Messages.getString("LongDurationField.1") : " ") + nhours + ":" + FMT.format(nminutes); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		} else {
			return nhours + ":" + FMT.format(nminutes); //$NON-NLS-1$
		}
	}
	
	protected abstract float getValue(LogEntry le); 


}
